/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *   * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.wso2.registry.secure;

import junit.framework.TestCase;
import org.wso2.registry.*;
import org.wso2.registry.jdbc.JDBCRegistry;
import org.wso2.usermanager.Realm;

public class SecureRegistryTest extends TestCase {

    private static Registry registry = null;
    private static Realm realm = null;

    public void setUp() {
        try {
            if (registry == null) {
                //String sqlFilePath = "modules/core/src/test/resources/um.sql";
                String sqlFilePath = "src/test/resources/um.sql";
                realm = new RegistryRealmFactory().newInMemoryRegistryRealm(sqlFilePath);
                registry = new JDBCRegistry(realm);
            }
        } catch (RegistryException e) {
            fail("Failed to initialize the registry.");
        }
    }

    public void testUserBasedAuthorization() throws RegistryException {

        SecureRegistry adminRegistry =
                new SecureRegistry(RegistryConstants.ADMIN_USER, "admin", registry, realm);
        RegistryUserManager adminUserManager =
                new RegistryUserManager(realm, RegistryConstants.ADMIN_USER);

        String r1Content = "R1";
        Artifact r1 = new Artifact();
        r1.setContent(r1Content.getBytes());

        try {
            adminRegistry.put("/r1", r1);
        } catch (RegistryException e) {
            fail("Valid put scenario failed.");
        }

        adminUserManager.addUser("chathura", "cce");
        adminUserManager.addUser("upul", "u" );

        adminUserManager.authorizeUser("chathura", "/r1", ActionConstants.GET);

        SecureRegistry chathuraRegistry =
                new SecureRegistry("chathura", "cce", registry, realm);

        try {
            chathuraRegistry.get("/r1");
        } catch (AuthorizationFailedException e) {
            fail("Could not get authorized resource.");
        }

        SecureRegistry upulRegistry =
                new SecureRegistry("upul", "u", registry, realm);

        boolean failed = false;
        try {
            upulRegistry.get("/r1");
        } catch (AuthorizationFailedException e) {
            failed = true;
        }

        assertTrue("Allowed to get unauthorized resource.", failed);
    }

    //public void testRoleBasedAuthorization() throws RegistryException {

        //SecureRegistry adminRegistry =
        //        new SecureRegistry(RegistryConstants.ADMIN_USER, "admin", registry, realm);
        //RegistryUserManager adminUserManager =
        //        new RegistryUserManager(realm, RegistryConstants.ADMIN_USER);
        //
        //String r1Content = "R1";
        //Artifact r1 = new Artifact();
        //r1.setContent(r1Content.getBytes());
        //
        //try {
        //    adminRegistry.put("/r1", r1);
        //} catch (RegistryException e) {
        //    fail("Valid put scenario failed.");
        //}
        //
        //adminUserManager.addRole("registryTeam");
        //adminUserManager.authorizeRole("registryTeam", "/r1", ActionConstants.GET);
        //
        //adminUserManager.addUser("chathura", "cce");
        //adminUserManager.addUserToRole("chathura", "registryTeam");
        //
        //adminUserManager.addUser("upul", "u");
        //
        //SecureRegistry chathuraRegistry =
        //        new SecureRegistry("chathura", "cce", registry, realm);
        //
        //try {
        //    chathuraRegistry.get("/r1");
        //} catch (AuthorizationFailedException e) {
        //    fail("Could not get authorized resource.");
        //}
        //
        //SecureRegistry upulRegistry =
        //        new SecureRegistry("upul", "u", registry, realm);
        //
        //boolean failed = false;
        //try {
        //    upulRegistry.get("/r1");
        //} catch (AuthorizationFailedException e) {
        //    failed = true;
        //}
        //
        //assertTrue("Allowed to get unauthorized resource.", failed);
    //}

    //public void putResourceTest() {
    //
    //    try {
    //        RegistryRealmFactory realmFactory = new RegistryRealmFactory();
    //        Realm realm = realmFactory.newRegistryRealm();
    //        RegistryUserManager userManager = new RegistryUserManager(realm);
    //
    //        SecureRegistry adminRegistry = new SecureRegistry(
    //                RegistryConstants.ADMIN_USER, "admin", registry, realm);
    //
    //        //userManager.addUser("chathura", "cce123", RegistryConstants.ADMIN_USER);
    //        SecureRegistry chathuraRegistry = new SecureRegistry(
    //                "chathura", "cce123", registry, realm);
    //
    //        Artifact a1 = new Artifact();
    //        a1.setContent("test".getBytes());
    //        adminRegistry.put("/d1/d2/d3/r1", a1);
    //
    //        Artifact a2 = new Artifact();
    //        a2.setContent("chathura's content".getBytes());
    //
    //        boolean failed = false;
    //        try {
    //            chathuraRegistry.put("/d1/d2/d4/r2", a2);
    //        } catch (RegistryException e) {
    //            failed = true;
    //        }
    //        assertTrue("Allowed put a resource to an unauthorized path.", failed);
    //
    //        userManager.authorizeUser(
    //                "chathura", "/d1/d2", ActionConstants.PUT, RegistryConstants.ADMIN_USER);
    //
    //        try {
    //            chathuraRegistry.put("/d1/d2/d4/r2", a2);
    //        } catch (RegistryException e) {
    //            fail("Denied to put a resource to an authorized path.");
    //        }
    //
    //    } catch (RegistryException e) {
    //        e.printStackTrace();
    //    }
    //}
}
